import time
from typing import Callable

from dicgram.mensagem import Mensagem


def polling_message(func: Callable) -> Callable:
    """
    Decorator para verificar novas mensagens

    :param func: função a ser decorada
    :return: função decorada
    """

    def wrapper(*args, **kwargs) -> None:
        """
        Wrapper da função decorada

        :param args: argumentos da função decorada
        :param kwargs: argumentos nomeados da função decorada
        :return: None
        """

        self = args[0]

        last_update_id = None
        while True:
            updates = self._get_updates(last_update_id)
            if not updates:
                pass
            elif 'result' in updates:
                for result in updates['result']:
                    if result['update_id'] != last_update_id:
                        last_update_id = result['update_id']
                        func(*args, **kwargs, msg=Mensagem(result))
            time.sleep(self._polling_rate)

    return wrapper


def webhook_message(func: Callable) -> Callable:
    """
    Decorator para verificar novas mensagens

    :param func: função a ser decorada
    :return: função decorada
    """

    def wrapper(*args, **kwargs) -> None:
        """
        Wrapper da função decorada

        :param args: argumentos da função decorada
        :param kwargs: argumentos nomeados da função decorada
        :return: None
        """

        self = args[0]
        self._set_webhook(self._webhook_url)

        from fastapi import FastAPI, Request
        from fastapi.responses import JSONResponse

        app = FastAPI()

        @app.post('/' + self._webhook_url.split('/')[-1])
        async def webhook(request: Request):
            """
            Webhook

            :param request: request
            :return: JSONResponse
            """

            data = await request.json()
            func(*args, **kwargs, msg=Mensagem(data))
            return JSONResponse({'status': 'ok'})

        import uvicorn

        uvicorn.run(app, host='0.0.0.0', port=self._webhook_port, log_level='error')

    return wrapper


def check_funcao_resp(func: Callable) -> Callable:
    """
    Decorator para verificar se a função de resposta é válida

    :param func: função a ser decorada
    :return: função decorada
    """

    def wrapper(*args, **kwargs) -> None:
        try:
            func(*args, **kwargs)
        except TypeError as e:
            if 'str' in str(e):
                raise Exception('Função de evento não definida')

    return wrapper



# Author: Marcelo Batista
# GitHub: https://github.com/marcellobatiista
# Sábado, 12 de Novembro de 2022


import json


class Campos:
    campos = ['message',
              'edited_message', 'channel_post',
              'edited_channel_post', 'inline_query',
              'chosen_inline_result', 'callback_query',
              'shipping_query', 'pre_checkout_query',
              'poll', 'poll_answer',
              'my_chat_member', 'chat_member',
              'chat_join_request']

    def __init__(self, message: dict):
        self.message = message

    def update(self) -> int:
        """
        Retorna o campo da mensagem

        :return: str
        """

        return list(self.message.keys())[1]


class Mensagem:
    """
    Classe que transforma um dicionário em um objeto,
    """

    def __init__(self, dicionario: dict):
        """
        :param dicionario: dicionário a ser transformado em objeto
        """

        self.update = None

        for k, v in dicionario.items():
            if isinstance(v, dict):
                setattr(self, k.replace('from', 'from_user'), Mensagem(v))
            else:
                if k == 'update_id':
                    setattr(self, k.replace('update_id', 'update'), Campos(dicionario).update())
                else:
                    setattr(self, k.replace('from', 'from_user'), v)

    def __str__(self):
        """
        Existem casos em que o valor do atributo é um objeto,
        então é necessário transformar o objeto em um dicionário
        para que possa ser transformado em uma string
        """

        return json.dumps(self.__dict__, default=lambda o: o.__dict__, indent=4)



from dicgram.cliente import Bot

copy = 'Copyright (c) 2022 Marcelo <https://github.com/marcellobatiista>'
version = '2.0.8'
autor = 'Marcelo Batista'
email = 'batista.marcelo34@gmail.com'

Bot.copyright = copy
Bot.version = version



import unittest

from dicgram.metodos import Metodos


class MeusCasosDeTestes(unittest.TestCase):
    def teste_todos_os_metodos_sem_traco(self):
        for mtd in Metodos._get_metodos():
            self.assertFalse('-' in mtd)

    def teste_todos_os_metodos_sem_espaco(self):
        for mtd in Metodos._get_metodos():
            self.assertFalse(' ' in mtd)


if __name__ == '__main__':
    unittest.main()
